@@ -8,7 +8,7 @@
  *	New quota format implementation - Jan Kara <jack@suse.cz> - Sponsored by SuSE CR
  */
 
-#ident "$Id: quotacheck.c,v 1.47 2005/06/24 14:04:23 jkar8572 Exp $"
+#ident "$Id: quotacheck.c,v 1.51 2006/05/13 01:05:24 jkar8572 Exp $"
 
 #include <dirent.h>
 #include <stdio.h>
@@ -275,8 +275,14 @@ static inline void blit(char *msg)
 	static int slow_down;
 
 	if (flags & FL_VERYVERBOSE && msg) {
+		int len = strlen(msg);
+		
 		putchar('\r');
-		printf("%-70s ", msg);
+		printf("%.70s", msg);
+		if (len > 70)
+			fputs("...", stdout);
+		else
+			printf("%*s",73-len, "");
 	}
 	if (flags & FL_VERYVERBOSE || ++slow_down >= BLIT_RATIO) {
 		putchar(bits[bitc]);
@@ -290,7 +296,24 @@ static inline void blit(char *msg)
 
 static void usage(void)
 {
-	printf(_("Utility for checking and repairing quota files.\n%s [-gucfinvdmMR] [-F <quota-format>] filesystem|-a\n"), progname);
+	printf(_("Utility for checking and repairing quota files.\n%s [-gucbfinvdmMR] [-F <quota-format>] filesystem|-a\n\n\
+-u, --user                check user files\n\
+-g, --group               check group files\n\
+-c, --create-files        create new quota files\n\
+-b, --backup              create backups of old quota files\n\
+-f, --force               force check even if quotas are enabled\n\
+-i, --interactive         interactive mode\n\
+-n, --use-first-dquot     use the first copy of duplicated structure\n\
+-v, --verbose             print more information\n\
+-d, --debug               print even more messages\n\
+-m, --no-remount          do not remount filesystem read-only\n\
+-M, --try-remount         try remounting filesystem read-only,\n\
+                          continue even if it fails\n\
+-R, --exclude-root        exclude root when checking all filesystems\n\
+-F, --format=formatname   check quota files of specific format\n\
+-a, --all                 check all filesystems\n\
+-h, --help                display this message and exit\n\
+-V, --version             display version information and exit\n\n"), progname);
 	printf(_("Bugs to %s\n"), MY_EMAIL);
 	exit(1);
 }
@@ -298,8 +321,27 @@ static void usage(void)
 static void parse_options(int argcnt, char **argstr)
 {
 	int ret;
+	struct option long_opts[] = {
+		{ "version", 0, NULL, 'V' },
+		{ "help", 0, NULL, 'h' },
+		{ "backup", 0, NULL, 'b' },
+		{ "create-files", 0, NULL, 'c' },
+		{ "verbose", 0, NULL, 'v' },
+		{ "debug", 0, NULL, 'd' },
+		{ "user", 0, NULL, 'u' },
+		{ "group", 0, NULL, 'g' },
+		{ "interactive", 0, NULL, 'i' },
+		{ "use-first-dquot", 0, NULL, 'n' },
+		{ "force", 0, NULL, 'f' },
+		{ "format", 1, NULL, 'F' },
+		{ "no-remount", 0, NULL, 'm' },
+		{ "try-remount", 0, NULL, 'M' },
+		{ "exclude-root", 0, NULL, 'R' },
+		{ "all", 0, NULL, 'a' },
+		{ NULL, 0, NULL, 0 }
+	};
 
-	while ((ret = getopt(argcnt, argstr, "VhbcvugidnfF:mMRa")) != -1) {
+	while ((ret = getopt_long(argcnt, argstr, "VhbcvugidnfF:mMRa", long_opts, NULL)) != -1) {
   	        switch (ret) {
 		  case 'b':
   		          flags |= FL_BACKUPS;
@@ -840,7 +882,7 @@ static void check_dir(struct mntent *mnt)
 	if (lstat(mnt->mnt_dir, &st) < 0)
 		die(2, _("Cannot stat mountpoint %s: %s\n"), mnt->mnt_dir, strerror(errno));
 	if (!S_ISDIR(st.st_mode))
-		die(2, _("Mountpoint %s isn't a directory?!\n"), mnt->mnt_dir);
+		die(2, _("Mountpoint %s is not a directory?!\n"), mnt->mnt_dir);
 	cur_dev = st.st_dev;
 	files_done = dirs_done = 0;
 	if (ucheck)
@@ -889,6 +931,8 @@ Please stop all programs writing to filesystem or use -m flag to force checking.
 			goto out;
 	}
 	dirs_done++;
+	if (flags & FL_VERBOSE || flags & FL_VERYVERBOSE)
+		fputs(_("done\n"), stdout);
 	if (ucheck) {
 		sub_quota_file(mnt, USRQUOTA, USRQUOTA);
 		sub_quota_file(mnt, USRQUOTA, GRPQUOTA);
@@ -897,8 +941,6 @@ Please stop all programs writing to filesystem or use -m flag to force checking.
 		sub_quota_file(mnt, GRPQUOTA, USRQUOTA);
 		sub_quota_file(mnt, GRPQUOTA, GRPQUOTA);
 	}
-	if (flags & FL_VERBOSE)
-		fputs(_("done\n"), stdout);
 	debug(FL_DEBUG | FL_VERBOSE, _("Checked %d directories and %d files\n"), dirs_done,
 	      files_done);
 	if (remounted) {
@@ -959,7 +1001,7 @@ jquota_err:
 			space = option + strlen(option);
 		if (space-option > sizeof(fmtbuf))
 			goto jquota_err;
-		sstrncpy(fmtbuf, option+1, space-option+1);
+		sstrncpy(fmtbuf, option+1, space-option);
 		if ((fmt = name2fmt(fmtbuf)) == QF_ERROR)
 			goto jquota_err;
 		return fmt;
@@ -988,7 +1030,7 @@ static void check_all(void)
 	int checked = 0;
 
 	if (init_mounts_scan((flags & FL_ALL) ? 0 : 1, &mntpoint, 0) < 0)
-		die(2, _("Can't initialize mountpoint scan.\n"));
+		die(2, _("Cannot initialize mountpoint scan.\n"));
 	while ((mnt = get_next_mount())) {
 		if (flags & FL_ALL && flags & FL_NOROOT && !strcmp(mnt->mnt_dir, "/"))
 			continue;
@@ -1020,7 +1062,7 @@ static void check_all(void)
 	}
 	end_mounts_scan();
 	if (!checked && (!(flags & FL_ALL) || flags & (FL_VERBOSE | FL_DEBUG)))
-		errstr(_("Can't find filesystem to check or filesystem not mounted with quota option.\n"));
+		errstr(_("Cannot find filesystem to check or filesystem not mounted with quota option.\n"));
 }
 
 int main(int argc, char **argv)
