@@ -34,7 +34,7 @@
 
 #ident "$Copyright: (c) 1980, 1990 Regents of the University of California $"
 #ident "$Copyright: All rights reserved. $"
-#ident "$Id: quotaon.c,v 1.20 2005/06/01 07:20:50 jkar8572 Exp $"
+#ident "$Id: quotaon.c,v 1.22 2006/05/13 01:05:24 jkar8572 Exp $"
 
 /*
  * Turn quota on/off for a filesystem.
@@ -64,15 +64,39 @@ char *xarg = NULL;
 
 static void usage(void)
 {
-	errstr(_("Usage:\n\t%s [-guvp] [-F quotaformat] [-x state] -a\n\t%s [-guvp] [-F quotaformat] [-x state] filesys ...\n"), progname, progname);
+	errstr(_("Usage:\n\t%s [-guvp] [-F quotaformat] [-x state] -a\n\
+\t%s [-guvp] [-F quotaformat] [-x state] filesys ...\n\n\
+-a, --all                turn quotas on for all filesystems\n\
+-f, --off                turn quotas off\n\
+-u, --user               operate on user quotas\n\
+-g, --group              operate on group quotas\n\
+-p, --print-state        print whether quotas are on or off\n\
+-x, --xfs-command=cmd    perform XFS quota command\n\
+-F, --format=formatname  operate on specific quota format\n\
+-v, --verbose            print more messages\n\
+-h, --help               display this help text and exit\n\
+-V, --version            display version information and exit\n"), progname, progname);
 	exit(1);
 }
 
 static void parse_options(int argcnt, char **argstr)
 {
 	int c;
+	struct option long_opts[] = {
+		{ "all", 0, NULL, 'a' },
+		{ "off", 0, NULL, 'f' },
+		{ "verbose", 0, NULL, 'v' },
+		{ "user", 0, NULL, 'u' },
+		{ "group", 0, NULL, 'g' },
+		{ "print-state", 0, NULL, 'p' },
+		{ "xfs-command", 1, NULL, 'x' },
+		{ "format", 1, NULL, 'F' },
+		{ "version", 0, NULL, 'V' },
+		{ "help", 0, NULL, 'h' },
+		{ NULL, 0, NULL, 0 }
+	};
 
-	while ((c = getopt(argcnt, argstr, "afvugpx:VF:")) != -1) {
+	while ((c = getopt_long(argcnt, argstr, "afvugpx:VF:h", long_opts, NULL)) != -1) {
 		switch (c) {
 		  case 'a':
 			  flags |= FL_ALL;
@@ -102,6 +126,7 @@ static void parse_options(int argcnt, char **argstr)
 		  case 'V':
 			  version();
 			  exit(0);
+		  case 'h':
 		  default:
 			  usage();
 		}
@@ -111,7 +136,7 @@ static void parse_options(int argcnt, char **argstr)
 		usage();
 	}
 	if (fmt == QF_RPC) {
-		fputs(_("Can't turn on/off quotas via RPC.\n"), stderr);
+		fputs(_("Cannot turn on/off quotas via RPC.\n"), stderr);
 		exit(1);
 	}
 	if (!(flags & (FL_USER | FL_GROUP)))
@@ -139,7 +164,7 @@ static int newstate(struct mntent *mnt, int type, char *extra)
 
 	if (!strcmp(mnt->mnt_type, MNTTYPE_XFS)) {	/* XFS filesystem has special handling... */
 		if (!(kernel_formats & (1 << QF_XFS))) {
-			errstr(_("Can't change state of XFS quota. It's not compiled in kernel.\n"));
+			errstr(_("Cannot change state of XFS quota. It's not compiled in kernel.\n"));
 			return 1;
 		}
 		if (kernel_formats & (1 << QF_XFS) &&
@@ -216,7 +241,7 @@ static int quotaonoff(char *quotadev, char *quotadir, char *quotafile, int type,
 	}
 	if (quotactl(qcmd, quotadev, kqf, (void *)quotafile) < 0) {
 		if (errno == ENOENT)
-			errstr(_("can't find %s on %s [%s]\n"), quotafile, quotadev, quotadir);
+			errstr(_("cannot find %s on %s [%s]\n"), quotafile, quotadev, quotadir);
 		else
 			errstr(_("using %s on %s [%s]: %s\n"), quotafile, quotadev, quotadir, strerror(errno));
 		if (errno == EINVAL)
@@ -326,7 +351,7 @@ int main(int argc, char **argv)
 	while ((mnt = get_next_mount())) {
 		if (nfs_fstype(mnt->mnt_type)) {
 			if (!(flags & FL_ALL))
-				fprintf(stderr, "%s: Quota can't be turned on on NFS filesystem\n", mnt->mnt_fsname);
+				fprintf(stderr, "%s: Quota cannot be turned on on NFS filesystem\n", mnt->mnt_fsname);
 			continue;
 		}
 
