@@ -34,7 +34,7 @@
 
 #ident "$Copyright: (c) 1980, 1990 Regents of the University of California. $"
 #ident "$Copyright: All rights reserved. $"
-#ident "$Id: quotaops.c,v 1.14 2004/05/24 19:39:15 jkar8572 Exp $"
+#ident "$Id: quotaops.c,v 1.20 2006/07/21 15:14:01 jkar8572 Exp $"
 
 #include <rpc/rpc.h>
 #include <sys/types.h>
@@ -72,13 +72,13 @@ void update_grace_times(struct dquot *q)
 	time_t now;
 
 	time(&now);
-	if (q->dq_dqb.dqb_bsoftlimit && toqb(q->dq_dqb.dqb_curspace) >= q->dq_dqb.dqb_bsoftlimit) {
+	if (q->dq_dqb.dqb_bsoftlimit && toqb(q->dq_dqb.dqb_curspace) > q->dq_dqb.dqb_bsoftlimit) {
 		if (!q->dq_dqb.dqb_btime)
 			q->dq_dqb.dqb_btime = now + q->dq_h->qh_info.dqi_bgrace;
 	}
 	else
 		q->dq_dqb.dqb_btime = 0;
-	if (q->dq_dqb.dqb_isoftlimit && q->dq_dqb.dqb_curinodes >= q->dq_dqb.dqb_isoftlimit) {
+	if (q->dq_dqb.dqb_isoftlimit && q->dq_dqb.dqb_curinodes > q->dq_dqb.dqb_isoftlimit) {
 		if (!q->dq_dqb.dqb_itime)
 			q->dq_dqb.dqb_itime = now + q->dq_h->qh_info.dqi_igrace;
 	}
@@ -118,7 +118,8 @@ struct dquot *getprivs(qid_t id, struct quota_handle **handles, int quiet)
 				if (ngroups > NGROUPS) {
 					gidsetp = malloc(ngroups * sizeof(gid_t));
 					if (!gidsetp) {
-						errstr(_("%s: gid set allocation (%d): %s\n"), name, ngroups, strerror(errno));
+						gid2group(id, name);
+						errstr(_("%s (gid %d): gid set allocation (%d): %s\n"), name, id, ngroups, strerror(errno));
 						return (struct dquot *)NULL;
 					}
 				}
@@ -128,7 +129,8 @@ struct dquot *getprivs(qid_t id, struct quota_handle **handles, int quiet)
 				if (ngroups < 0) {
 					if (gidsetp != gidset)
 						free(gidsetp);
-					errstr(_("%s: error while trying getgroups(): %s\n"), name, strerror(errno));
+					gid2group(id, name);
+					errstr(_("%s (gid %d): error while trying getgroups(): %s\n"), name, id, strerror(errno));
 					return (struct dquot *)NULL;
 				}
 
@@ -151,9 +153,11 @@ struct dquot *getprivs(qid_t id, struct quota_handle **handles, int quiet)
 
 		if (!(q = handles[i]->qh_ops->read_dquot(handles[i], id))) {
 			/* If rpc.rquotad is not running filesystem might be just without quotas... */
-			if (errno != ENOENT && (errno != ECONNREFUSED || !quiet))
-				errstr(_("%s: error while getting quota from %s for %u: %s\n"),
-					name, handles[i]->qh_quotadev, id, strerror(errno));
+			if (errno != ENOENT && (errno != ECONNREFUSED || !quiet)) {
+				id2name(id, handles[i]->qh_type, name);
+				errstr(_("error while getting quota from %s for %s (id %u): %s\n"),
+					handles[i]->qh_quotadev, name, id, strerror(errno));
+			}
 			continue;
 		}
 		if (qhead == NULL)
@@ -176,7 +180,7 @@ int putprivs(struct dquot *qlist, int flags)
 
 	for (q = qlist; q; q = q->dq_next) {
 		if (q->dq_h->qh_ops->commit_dquot(q, flags) == -1) {
-			errstr(_("Can't write quota for %u on %s: %s\n"),
+			errstr(_("Cannot write quota for %u on %s: %s\n"),
 				q->dq_id, q->dq_h->qh_quotadev, strerror(errno));
 			ret = -1;
 			continue;
@@ -201,7 +205,7 @@ int editprivs(char *tmpfile)
 	sigaddset(&nmask, SIGHUP);
 	sigprocmask(SIG_SETMASK, &nmask, &omask);
 	if ((pid = fork()) < 0) {
-		errstr("Can't fork(): %s\n", strerror(errno));
+		errstr("Cannot fork(): %s\n", strerror(errno));
 		return -1;
 	}
 	if (pid == 0) {
@@ -233,7 +237,7 @@ int editprivs(char *tmpfile)
 		edpars[i++] = tmpfile;
 		edpars[i] = NULL;
 		execvp(edpars[0], edpars);
-		die(1, _("Can't exec %s\n"), ed);
+		die(1, _("Cannot exec %s\n"), ed);
 	}
 	waitpid(pid, &stat, 0);
 	sigprocmask(SIG_SETMASK, &omask, NULL);
@@ -252,11 +256,11 @@ int writeprivs(struct dquot *qlist, int outfd, char *name, int quotatype)
 	ftruncate(outfd, 0);
 	lseek(outfd, 0, SEEK_SET);
 	if (!(fd = fdopen(dup(outfd), "w")))
-		die(1, _("Can't duplicate descriptor of file to write to: %s\n"), strerror(errno));
+		die(1, _("Cannot duplicate descriptor of file to write to: %s\n"), strerror(errno));
 
 #if defined(ALT_FORMAT)
 	fprintf(fd, _("Disk quotas for %s %s (%cid %d):\n"),
-		type2name(quotatype), name, *type2name(quotatype), name2id(name, quotatype, NULL));
+		type2name(quotatype), name, *type2name(quotatype), qlist->dq_id);
 
 	fprintf(fd,
 		_("  Filesystem                   blocks       soft       hard     inodes     soft     hard\n"));
@@ -330,7 +334,7 @@ int readprivs(struct dquot *qlist, int infd)
 
 	lseek(infd, 0, SEEK_SET);
 	if (!(fd = fdopen(dup(infd), "r")))
-		die(1, _("Can't duplicate descriptor of temp file: %s\n"), strerror(errno));
+		die(1, _("Cannot duplicate descriptor of temp file: %s\n"), strerror(errno));
 
 #if defined(ALT_FORMAT)
 	/*
@@ -437,10 +441,10 @@ int writeindividualtimes(struct dquot *qlist, int outfd, char *name, int quotaty
 	ftruncate(outfd, 0);
 	lseek(outfd, 0, SEEK_SET);
 	if (!(fd = fdopen(dup(outfd), "w")))
-		die(1, _("Can't duplicate descriptor of file to write to: %s\n"), strerror(errno));
+		die(1, _("Cannot duplicate descriptor of file to write to: %s\n"), strerror(errno));
 
 	fprintf(fd, _("Times to enforce softlimit for %s %s (%cid %d):\n"),
-		type2name(quotatype), name, *type2name(quotatype), name2id(name, quotatype, NULL));
+		type2name(quotatype), name, *type2name(quotatype), qlist->dq_id);
 	fprintf(fd, _("Time units may be: days, hours, minutes, or seconds\n"));
 	fprintf(fd,
 		_("  Filesystem                         block grace               inode grace\n"));
@@ -479,7 +483,7 @@ int readindividualtimes(struct dquot *qlist, int infd)
 
 	lseek(infd, 0, SEEK_SET);
 	if (!(fd = fdopen(dup(infd), "r")))
-		die(1, _("Can't duplicate descriptor of temp file: %s\n"), strerror(errno));
+		die(1, _("Cannot duplicate descriptor of temp file: %s\n"), strerror(errno));
 
 	/*
 	 * Discard title lines, then read lines to process.
@@ -503,7 +507,7 @@ format_err:
 				goto format_err;
 			if (str2timeunits(btime, bunits, &bseconds) < 0) {
 units_err:
-				errstr(_("Bad time units. Units are 'second', 'minute', 'hour' and 'day'.\n"));
+				errstr(_("Bad time units. Units are 'second', 'minute', 'hour', and 'day'.\n"));
 				return -1;
 			}
 			bseconds += now;
@@ -539,7 +543,7 @@ int writetimes(struct quota_handle **handles, int outfd)
 	ftruncate(outfd, 0);
 	lseek(outfd, 0, SEEK_SET);
 	if ((fd = fdopen(dup(outfd), "w")) == NULL)
-		die(1, _("Can't duplicate descriptor of file to edit: %s\n"), strerror(errno));
+		die(1, _("Cannot duplicate descriptor of file to edit: %s\n"), strerror(errno));
 
 #if defined(ALT_FORMAT)
 	fprintf(fd, _("Grace period before enforcing soft limits for %ss:\n"),
@@ -587,7 +591,7 @@ int readtimes(struct quota_handle **handles, int infd)
 		return 0;
 	lseek(infd, 0, SEEK_SET);
 	if (!(fd = fdopen(dup(infd), "r"))) {
-		errstr(_("Can't reopen temp file: %s\n"),
+		errstr(_("Cannot reopen temp file: %s\n"),
 			strerror(errno));
 		return -1;
 	}
@@ -639,7 +643,7 @@ int readtimes(struct quota_handle **handles, int infd)
 #endif
 		if (str2timeunits(btime, bunits, &bseconds) < 0 ||
 		    str2timeunits(itime, iunits, &iseconds) < 0) {
-			errstr(_("Bad time units. Units are 'second', 'minute', 'hour' and 'day'.\n"));
+			errstr(_("Bad time units. Units are 'second', 'minute', 'hour', and 'day'.\n"));
 			return -1;
 		}
 		for (i = 0; handles[i]; i++) {
