@@ -10,7 +10,7 @@
  * 
  * Author:  Marco van Wieringen <mvw@planets.elm.net>
  *
- * Version: $Id: warnquota.c,v 1.22 2005/06/24 14:04:23 jkar8572 Exp $
+ * Version: $Id: warnquota.c,v 1.26 2006/05/13 01:05:24 jkar8572 Exp $
  *
  *          This program is free software; you can redistribute it and/or
  *          modify it under the terms of the GNU General Public License as
@@ -27,9 +27,11 @@
 #include <ctype.h>
 #include <signal.h>
 #include <grp.h>
+#include <time.h>
+#include <getopt.h>
 #include <sys/types.h>
 #include <sys/wait.h>
-#include <time.h>
+#include <sys/utsname.h>
 #ifdef USE_LDAP_MAIL_LOOKUP
 #include <ldap.h>
 #endif
@@ -90,7 +92,7 @@ struct usage {
 };
 
 #ifdef USE_LDAP_MAIL_LOOKUP
-LDAP *ldapconn = NULL;
+static LDAP *ldapconn = NULL;
 #endif
 
 struct configparams {
@@ -137,14 +139,15 @@ struct adminstable {
 	char *adminname;
 };
 
-int qtab_i = 0, fmt = -1, flags;
-char maildev[CNF_BUFFER];
-struct quota_handle *maildev_handle;
-char *configfile = WARNQUOTA_CONF, *quotatabfile = QUOTATAB, *adminsfile = ADMINSFILE;
+static int qtab_i = 0, fmt = -1, flags;
+static char maildev[CNF_BUFFER];
+static struct quota_handle *maildev_handle;
+static char *configfile = WARNQUOTA_CONF, *quotatabfile = QUOTATAB, *adminsfile = ADMINSFILE;
 char *progname;
-quotatable_t *quotatable;
-int adminscnt, adminsalloc;
-struct adminstable *adminstable;
+static char *hostname, *domainname;
+static quotatable_t *quotatable;
+static int adminscnt, adminsalloc;
+static struct adminstable *adminstable;
 
 /*
  * Global pointers to list.
@@ -184,14 +187,14 @@ static int setup_ldap(struct configparams *config)
 		
 #endif
 
-struct offenderlist *add_offender(int type, int id, char *name)
+static struct offenderlist *add_offender(int type, int id, char *name)
 {
 	struct offenderlist *offender;
 	char namebuf[MAXNAMELEN];
 	
 	if (!name) {
 		if (id2name(id, type, namebuf)) {
-			errstr(_("Can't get name for uid/gid %u.\n"), id);
+			errstr(_("Cannot get name for uid/gid %u.\n"), id);
 			return NULL;
 		}
 		name = namebuf;
@@ -206,7 +209,7 @@ struct offenderlist *add_offender(int type, int id, char *name)
 	return offender;
 }
 
-void add_offence(struct dquot *dquot, char *name)
+static void add_offence(struct dquot *dquot, char *name)
 {
 	struct offenderlist *lptr;
 	struct usage *usage;
@@ -230,7 +233,7 @@ void add_offence(struct dquot *dquot, char *name)
 	lptr->usage = usage;
 }
 
-int deliverable(struct dquot *dquot)
+static int deliverable(struct dquot *dquot)
 {
 	time_t now;
 	struct dquot *mdquot;
@@ -259,7 +262,7 @@ int deliverable(struct dquot *dquot)
 	return 1;
 }
 
-int check_offence(struct dquot *dquot, char *name)
+static int check_offence(struct dquot *dquot, char *name)
 {
 	if ((dquot->dq_dqb.dqb_bsoftlimit && toqb(dquot->dq_dqb.dqb_curspace) >= dquot->dq_dqb.dqb_bsoftlimit)
 	    || (dquot->dq_dqb.dqb_isoftlimit && dquot->dq_dqb.dqb_curinodes >= dquot->dq_dqb.dqb_isoftlimit)) {
@@ -269,43 +272,43 @@ int check_offence(struct dquot *dquot, char *name)
 	return 0;
 }
 
-FILE *run_mailer(char *command)
+static FILE *run_mailer(char *command)
 {
 	int pipefd[2];
 	FILE *f;
 
 	if (pipe(pipefd) < 0) {
-		errstr(_("Can't create pipe: %s\n"), strerror(errno));
+		errstr(_("Cannot create pipe: %s\n"), strerror(errno));
 		return NULL;
 	}
 	signal(SIGPIPE, SIG_IGN);
 	switch(fork()) {
 		case -1:
-			errstr(_("Can't fork: %s\n"), strerror(errno));
+			errstr(_("Cannot fork: %s\n"), strerror(errno));
 			return NULL;
 		case 0:
 			close(pipefd[1]);
 			if (dup2(pipefd[0], 0) < 0) {
-				errstr(_("Can't duplicate descriptor: %s\n"), strerror(errno));
+				errstr(_("Cannot duplicate descriptor: %s\n"), strerror(errno));
 				wc_exit(1);
 			}			
 			execl(SHELL, SHELL, "-c", command, NULL);
-			errstr(_("Can't execute '%s': %s\n"), command, strerror(errno));
+			errstr(_("Cannot execute '%s': %s\n"), command, strerror(errno));
 			wc_exit(1);
 		default:
 			close(pipefd[0]);
 			if (!(f = fdopen(pipefd[1], "w")))
-				errstr(_("Can't open pine: %s\n"), strerror(errno));
+				errstr(_("Cannot open pine: %s\n"), strerror(errno));
 			return f;
 	}
 }
 
-int admin_name_cmp(const void *key, const void *mem)
+static int admin_name_cmp(const void *key, const void *mem)
 {
 	return strcmp(key, ((struct adminstable *)mem)->grpname);
 }
 
-int should_cc(struct offenderlist *offender, struct configparams *config)
+static int should_cc(struct offenderlist *offender, struct configparams *config)
 {
 	struct usage *lptr;
 	struct util_dqblk *dqb;
@@ -324,7 +327,35 @@ int should_cc(struct offenderlist *offender, struct configparams *config)
 	return 0;
 }
 
-int mail_user(struct offenderlist *offender, struct configparams *config)
+/* Substitute %s and %i for 'name' and %h for hostname */
+static void format_print(FILE *fp, char *fmt, char *name)
+{
+	char *ch, *lastch = fmt;
+
+	for (ch = strchr(fmt, '%'); ch; lastch = ch+2, ch = strchr(ch+2, '%')) {
+		*ch = 0;
+		fputs(lastch, fp);
+		*ch = '%';
+		switch (*(ch+1)) {
+			case 's':
+			case 'i':
+				fputs(name, fp);
+				break;
+			case 'h':
+				fputs(hostname, fp);
+				break;
+			case 'd':
+				fputs(domainname, fp);
+				break;
+			case '%':
+				fputc('%', fp);
+				break;
+		}
+	}
+	fputs(lastch, fp);
+}
+
+static int mail_user(struct offenderlist *offender, struct configparams *config)
 {
 	struct usage *lptr;
 	FILE *fp;
@@ -441,12 +472,12 @@ int mail_user(struct offenderlist *offender, struct configparams *config)
 
 	if (offender->offender_type == USRQUOTA)
 		if (config->user_message)
-			fputs(config->user_message, fp);
+			format_print(fp, config->user_message, offender->offender_name);
 		else
 			fputs(DEF_USER_MESSAGE, fp);
 	else
 		if (config->group_message)
-			fprintf(fp, config->group_message, offender->offender_name);
+			format_print(fp, config->group_message, offender->offender_name);
 		else
 			fprintf(fp, DEF_GROUP_MESSAGE, offender->offender_name);
 
@@ -491,24 +522,24 @@ int mail_user(struct offenderlist *offender, struct configparams *config)
 
 	if (offender->offender_type == USRQUOTA)
 		if (config->user_signature)
-			fputs(config->user_signature, fp);
+			format_print(fp, config->user_signature, offender->offender_name);
 		else
 			fprintf(fp, DEF_USER_SIGNATURE, config->support, config->phone);
 	else
 		if (config->group_signature)
-			fputs(config->group_signature, fp);
+			format_print(fp, config->group_signature, offender->offender_name);
 		else
 			fprintf(fp, DEF_GROUP_SIGNATURE, config->support, config->phone);
 	fclose(fp);
 	if (wait(&status) < 0)	/* Wait for mailer */
-		errstr(_("Can't wait for mailer: %s\n"), strerror(errno));
+		errstr(_("Cannot wait for mailer: %s\n"), strerror(errno));
 	else if (!WIFEXITED(status) || WEXITSTATUS(status) != 0)
 		errstr(_("Warning: Mailer exitted abnormally.\n"));
 
 	return 0;
 }
 
-int mail_to_offenders(struct configparams *config)
+static int mail_to_offenders(struct configparams *config)
 {
 	struct offenderlist *lptr;
 	int ret = 0;
@@ -524,7 +555,7 @@ int mail_to_offenders(struct configparams *config)
 /*
  * Wipe spaces, tabs, quotes and newlines from beginning and end of string
  */
-void stripstring(char **buff)
+static void stripstring(char **buff)
 {
 	int i;
 
@@ -541,7 +572,7 @@ void stripstring(char **buff)
 /*
  * Substitute '|' with end of lines
  */
-void create_eoln(char *buf)
+static void create_eoln(char *buf)
 {
 	char *colpos = buf;
 
@@ -552,7 +583,7 @@ void create_eoln(char *buf)
 /*
  * Read /etc/quotatab (description of devices for users)
  */
-int get_quotatable(void)
+static int get_quotatable(void)
 {
 	FILE *fp;
 	char buffer[IOBUF_SIZE], *colpos, *devname, *devdesc;
@@ -560,7 +591,7 @@ int get_quotatable(void)
 	struct stat st;
 
 	if (!(fp = fopen(quotatabfile, "r"))) {
-		errstr(_("Can't open %s: %s\nWill use device names.\n"), quotatabfile, strerror(errno));
+		errstr(_("Cannot open %s: %s\nWill use device names.\n"), quotatabfile, strerror(errno));
 		qtab_i = 0;
 		return 0;
 	}
@@ -583,7 +614,7 @@ int get_quotatable(void)
 		}
 		/* Parse line */
 		if (!(colpos = strchr(buffer, ':'))) {
-			errstr(_("Can't parse line %d in quotatab (missing ':')\n"), line);
+			errstr(_("Cannot parse line %d in quotatab (missing ':')\n"), line);
 			qtab_i--;
 			continue;
 		}
@@ -597,17 +628,37 @@ int get_quotatable(void)
 		create_eoln(quotatable[qtab_i].devdesc);
 
 		if (stat(quotatable[qtab_i].devname, &st) < 0)
-			errstr(_("Can't stat device %s (maybe typo in quotatab)\n"), quotatable[qtab_i].devname);
+			errstr(_("Cannot stat device %s (maybe typo in quotatab)\n"), quotatable[qtab_i].devname);
 	}
 	fclose(fp);
 	return 0;
 }
 
+/* Check correctness of the given format */
+static void verify_format(char *fmt, char *varname)
+{
+	char *ch;
+
+	for (ch = strchr(fmt, '%'); ch; ch = strchr(ch+2, '%')) {
+		switch (*(ch+1)) {
+			case 's':
+			case 'i':
+			case 'h':
+			case 'd':
+			case '%':
+				continue;
+			default:
+				die(1, _("Incorrect format string for variable %s.\n\
+Unrecognized expression %%%c.\n"), varname, *(ch+1));
+		}
+	}
+}
+
 /*
  * Reads config parameters from configfile
  * uses default values if errstr occurs
  */
-int readconfigfile(const char *filename, struct configparams *config)
+static int readconfigfile(const char *filename, struct configparams *config)
 {
 	FILE *fp;
 	char buff[IOBUF_SIZE];
@@ -633,7 +684,7 @@ int readconfigfile(const char *filename, struct configparams *config)
 #endif
 
 	if (!(fp = fopen(filename, "r"))) {
-		errstr(_("Can't open %s: %s\n"), filename, strerror(errno));
+		errstr(_("Cannot open %s: %s\n"), filename, strerror(errno));
 		return -1;
 	}
 
@@ -692,18 +743,22 @@ int readconfigfile(const char *filename, struct configparams *config)
 			} else if (!strcmp(var, "MESSAGE")) {
 				config->user_message = sstrdup(value);
 				create_eoln(config->user_message);
+				verify_format(config->user_message, "MESSAGE");
 			}
 			else if (!strcmp(var, "SIGNATURE")) {
 				config->user_signature = sstrdup(value);
 				create_eoln(config->user_signature);
+				verify_format(config->user_signature, "SIGNATURE");
 			}
 			else if (!strcmp(var, "GROUP_MESSAGE")) {
 				config->group_message = sstrdup(value);
 				create_eoln(config->group_message);
+				verify_format(config->group_message, "GROUP_MESSAGE");
 			}
 			else if (!strcmp(var, "GROUP_SIGNATURE")) {
 				config->group_signature = sstrdup(value);
 				create_eoln(config->group_signature);
+				verify_format(config->group_signature, "GROUP_SIGNATURE");
 			}
 			else if (!strcmp(var, "LDAP_MAIL")) {
 				if(strcasecmp(value, "true") == 0) 
@@ -753,20 +808,20 @@ cc_parse_err:
 	return 0;
 }
 
-int admin_cmp(const void *a1, const void *a2)
+static int admin_cmp(const void *a1, const void *a2)
 {
 	return strcmp(((struct adminstable *)a1)->grpname, ((struct adminstable *)a2)->grpname);
 }
 
 /* Get administrators of the groups */
-int get_groupadmins(void)
+static int get_groupadmins(void)
 {
 	FILE *f;
 	int line = 0;
 	char buffer[IOBUF_SIZE], *colpos, *grouppos, *endname, *adminpos;
 
 	if (!(f = fopen(adminsfile, "r"))) {
-		errstr(_("Can't open file with group administrators: %s\n"), strerror(errno));
+		errstr(_("Cannot open file with group administrators: %s\n"), strerror(errno));
 		return -1;
 	}
 	
@@ -781,7 +836,7 @@ int get_groupadmins(void)
 		/* Find splitting colon */
 		for (grouppos = colpos; *colpos && *colpos != ':'; colpos++);
 		if (!*colpos || grouppos == colpos) {
-			errstr(_("Parse error at line %d. Can't find end of group name.\n"), line);
+			errstr(_("Parse error at line %d. Cannot find end of group name.\n"), line);
 			continue;
 		}
 		/* Cut trailing spaces */
@@ -790,7 +845,7 @@ int get_groupadmins(void)
 		/* Skip initial spaces at admins name */
 		for (colpos++; isspace(*colpos); colpos++);
 		if (!*colpos) {
-			errstr(_("Parse error at line %d. Can't find administrators name.\n"), line);
+			errstr(_("Parse error at line %d. Cannot find administrators name.\n"), line);
 			continue;
 		}
 		/* Go through admins name */
@@ -815,7 +870,7 @@ int get_groupadmins(void)
 	return 0;
 }
 
-struct quota_handle *find_handle_dev(char *dev, struct quota_handle **handles)
+static struct quota_handle *find_handle_dev(char *dev, struct quota_handle **handles)
 {
 	int i;
 
@@ -823,7 +878,7 @@ struct quota_handle *find_handle_dev(char *dev, struct quota_handle **handles)
 	return handles[i];
 }
 
-void warn_quota(void)
+static void warn_quota(void)
 {
 	struct quota_handle **handles;
 	struct configparams config;
@@ -835,7 +890,7 @@ void warn_quota(void)
 		wc_exit(1);
 
 	if (flags & FL_USER) {
-		handles = create_handle_list(0, NULL, USRQUOTA, -1, IOI_LOCALONLY | IOI_READONLY | IOI_OPENFILE, (flags & FL_NOAUTOFS ? MS_NO_AUTOFS : 0));
+		handles = create_handle_list(0, NULL, USRQUOTA, -1, IOI_READONLY | IOI_OPENFILE, MS_LOCALONLY | (flags & FL_NOAUTOFS ? MS_NO_AUTOFS : 0));
 		if (!maildev[0] || !strcasecmp(maildev, "any"))
 			maildev_handle = NULL;
 		else
@@ -847,7 +902,7 @@ void warn_quota(void)
 	if (flags & FL_GROUP) {
 		if (get_groupadmins() < 0)
 			wc_exit(1);
-		handles = create_handle_list(0, NULL, GRPQUOTA, -1, IOI_LOCALONLY | IOI_READONLY | IOI_OPENFILE, (flags & FL_NOAUTOFS ? MS_NO_AUTOFS : 0));
+		handles = create_handle_list(0, NULL, GRPQUOTA, -1, IOI_READONLY | IOI_OPENFILE, MS_LOCALONLY | (flags & FL_NOAUTOFS ? MS_NO_AUTOFS : 0));
 		if (!maildev[0] || !strcasecmp(maildev, "any"))
 			maildev_handle = NULL;
 		else
@@ -863,7 +918,18 @@ void warn_quota(void)
 /* Print usage information */
 static void usage(void)
 {
-	errstr(_("Usage:\n  warnquota [-ugsid] [-F quotaformat] [-c configfile] [-q quotatabfile]\n"));
+	errstr(_("Usage:\n  warnquota [-ugsid] [-F quotaformat] [-c configfile] [-q quotatabfile] [-a adminsfile]\n\n\
+-u, --user                      warn users\n\
+-g, --group                     warn groups\n\
+-s, --human-readable            send information in more human friendly units\n\
+-i, --no-autofs                 avoid autofs mountpoints\n\
+-d, --no-details                do not send quota information itself\n\
+-F, --format=formatname         use quotafiles of specific format\n\
+-c, --config=config-file        non-default config file\n\
+-q, --quota-tab=quotatab-file   non-default quotatab\n\
+-a, --admins-file=admins-file   non-default admins file\n\
+-h, --help                      display this help message and exit\n\
+-v, --version                   display version information and exit\n\n"));
 	fprintf(stderr, _("Bugs to %s\n"), MY_EMAIL);
 	wc_exit(1);
 }
@@ -871,15 +937,29 @@ static void usage(void)
 static void parse_options(int argcnt, char **argstr)
 {
 	int ret;
+	struct option long_opts[] = {
+		{ "user", 0, NULL, 'u' },
+		{ "group", 0, NULL, 'g' },
+		{ "version", 0, NULL, 'V' },
+		{ "help", 0, NULL, 'h' },
+		{ "format", 1, NULL, 'F' },
+		{ "config", 1, NULL, 'c' },
+		{ "quota-tab", 1, NULL, 'q' },
+		{ "admins-file", 1, NULL, 'a' },
+		{ "no-autofs", 0, NULL, 'i' },
+		{ "human-readable", 0, NULL, 's' },
+		{ "no-details", 0, NULL, 'd' },
+		{ NULL, 0, NULL, 0 }
+	};
  
-	while ((ret = getopt(argcnt, argstr, "ugVF:hc:q:a:isd")) != -1) {
+	while ((ret = getopt_long(argcnt, argstr, "ugVF:hc:q:a:isd", long_opts, NULL)) != -1) {
 		switch (ret) {
 		  case '?':
 		  case 'h':
 			usage();
 		  case 'V':
 			version();
-			break;
+			exit(0);
 		  case 'F':
 			if ((fmt = name2fmt(optarg)) == QF_ERROR)
 				wc_exit(1);
@@ -914,10 +994,21 @@ static void parse_options(int argcnt, char **argstr)
 		flags |= FL_USER;
 }
  
+static void get_host_name(void)
+{
+	struct utsname uts;
+
+	if (uname(&uts))
+		die(1, _("Cannot get host name: %s\n"), strerror(errno));
+	hostname = uts.nodename;
+	domainname = uts.domainname;
+}
+
 int main(int argc, char **argv)
 {
 	gettexton();
 	progname = basename(argv[0]);
+	get_host_name();
 
 	parse_options(argc, argv);
 	init_kernel_interface();
